--*******************************************************************************************************************************
--
-- Description: This module synchronizes channel processing to the selected start and stop events.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity synchronizer is
port (
    -- Clocks, reset, and state machine controls.
	clk 				: in std_logic;						-- PCI system clock from the Xilinx endpoint core.
	sample_clk 			: in std_logic;						-- Either the TX or RX clock depending on the type of channel.
	rst 				: in std_logic;						-- Active high asynchronous reset.
	synchronizer_status	: out std_logic_vector(1 downto 0);	-- Indicator bits for busy (0) and ready (1) status reporting in the PCI clock domain.
	tx_rx_select 		: in std_logic;						-- Configures the state machine for a transmit channel (0) or receive channel (1).

    -- Channel controls.
	synchronizer_on_off	: in std_logic_vector(1 downto 0);	-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	init_status			: in std_logic;						-- Logic one indicates that the datapath elements have been initialized.
	dma_enable 			: out std_logic;					-- Logic one enables the scatter gather controller.
	dma_go				: in std_logic;						-- Logic one indicates a burst transfer can be processed by the application logic.
	dma_busy			: in std_logic;						-- Logic one indicates that the DMA channel is currently active.
	stop_on_count		: out std_logic;					-- Logic one throttles the data after the requested number of output samples pass.
	count_done 			: in std_logic;						-- Logic one indicates that the requested number of output samples have passed.
	prime_done 			: in std_logic;						-- Logic one indicates that valid data is available at the end of the datapath.
	channel_active		: out std_logic;					-- Active high enable for the channel datapath.
	clear				: out std_logic;					-- Active high datapath clear.
	
    -- Channel start and stop commands.
	start_event			: in std_logic_vector(1 downto 0);	-- Select the event to activate the datapath after it is enabled. (none, software trigger, hardware trigger)
	stop_event			: in std_logic_vector(1 downto 0);	-- Select the event to deactivate the datapath. (software command, software trigger, hardware trigger, sample count)
	soft_trigger		: in std_logic;						-- Single period software trigger to be used by the application logic.
	hard_trigger		: in std_logic						-- Hardware rising edge start trigger and falling edge stop trigger.
);
end synchronizer;

architecture structural of synchronizer is

	type SYNCHRONIZER_STATE is (idle, initialize, prime, start_trig, stop_trig, purge, residue);	--	Type definitions for the state machine.
	signal ch_state 			: SYNCHRONIZER_STATE;			-- Current state.
	signal pause_counter		: unsigned(3 downto 0);			-- Counter to pause sixteen clock periods.
	signal on_off				: std_logic_vector(1 downto 0);	-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	signal synchronizer_off		: std_logic; 					-- Active high off switch that is only cleared by the idle state.
	signal synchronizer_busy	: std_logic;					-- Logic one indicates the synchronizer is busy.
	signal channel_ready		: std_logic; 					-- Logic one indicates that the channel is waiting for a start trigger.
	signal dma_go_rclk			: std_logic; 					-- Logic one indicates a burst transfer can be processed by the application logic (Reclocked).
	signal dma_busy_rclk		: std_logic; 					-- Logic one indicates that the DMA channel is currently active (Reclocked).
	signal dma_disable			: std_logic;					-- Logic one disables the DMA channel.
	signal start_event_rclk		: std_logic_vector(1 downto 0);	-- Select the event to activate the datapath after it is enabled (Reclocked).
	signal stop_event_rclk		: std_logic_vector(1 downto 0);	-- Select the event to deactivate the datapath (Reclocked).
	
begin

--************************************************* STATE MACHINE TRANSITIONS **************************************************

	process (sample_clk,rst)
    begin
		if rst='1' then
		
			start_event_rclk <= (others => '0');
			stop_event_rclk <= (others => '0');
			dma_go_rclk <= '0';
			dma_busy_rclk <= '0';
			ch_state <= idle;

        elsif rising_edge(sample_clk) then
        
			-- The following PCI registers must be reclocked so that they are coherent to the sample clock.
			start_event_rclk <= start_event;
			stop_event_rclk <= stop_event;
			dma_go_rclk <= dma_go;
			dma_busy_rclk <= dma_busy;

			case ch_state is
				-- Remain idle until the synchronizer "on" switch is detected.
				when idle =>
					-- Monitor the on switch for an active state.
					if synchronizer_on_off(0) = '1' then
						ch_state <= initialize;
					end if;
					
				-- Pause for a minimum sixteen periods to allow the FIFOs to initialize from reset and the DMA engine to initialize.
				-- Note: The DMA FIFO takes four clock periods in each domain to initialize from reset.
				-- Note: Proper operation assumes that the sample clock is no more than four times as fast as the PCI clock.
				when initialize =>
					-- Wait for the counter to expire and all datapath elements to initialize.
					if pause_counter >= x"F" and init_status = '1' then
						-- Transition to the prime state if this is a transmit channel.
						if tx_rx_select = '0' then
							ch_state <= prime;
						-- Transition to the start trigger state if this is an initialization reset.
						else
							ch_state <= start_trig;
						end if;
					end if;
					
				-- Prime the datapath of a transmit channel so that valid data is available before the channel is enabled.
				when prime =>
					-- Wait for the datapath to be primed with valid data before entering the start trigger state.
					if prime_done = '1' then
						ch_state <= start_trig;
					end if;
					
				-- Wait for the selected start trigger.
				when start_trig =>
					-- Halt the process if the synchronizer "off" switch is active.
					if synchronizer_off = '1' then
						ch_state <= idle;
					-- Note:  The following several lines monitor all of the conditions that could satisfy a start event.
					-- No trigger selected, start immediately.
					elsif (start_event_rclk = "00") or
					-- Wait for an active software trigger.
					(start_event_rclk = "01" and soft_trigger = '1') or
					-- Wait for the hardware trigger rising edge.
					(start_event_rclk = "10" and hard_trigger = '1') then
						ch_state <= stop_trig;
					end if;
					
				-- Wait for the selected stop trigger.
				when stop_trig =>
					-- Note:  The following several lines monitor all of the conditions that could satisfy a stop event.
					-- Halt the process if the synchronizer "off" switch is active.
					-- Note:  This is the only way to stop the channel if stop_event_rclk = "00".
					if synchronizer_off = '1' or
					-- Wait for an active software trigger.
					(stop_event_rclk = "01" and soft_trigger = '1') or
					-- Wait for the hardware trigger falling edge.
					(stop_event_rclk = "10" and hard_trigger = '0') or
					-- Wait to process the specified number of samples.
					(stop_event_rclk = "11" and count_done = '1') then
						-- Deactivate the channel.
						if tx_rx_select = '0' then
							ch_state <= residue;
						else
							ch_state <= purge;
						end if;
					end if;
					
				--	Pause for sixteen periods to allow the receiver data pipeline time to purge with the channel deactivated.
				when purge =>
					-- Wait for the counter to expire.
					if pause_counter = x"F" then
						ch_state <= residue;
					end if;
					
				--	Process any pending DMA FIFO operations.
				when residue =>
					-- Transition to idle only if there are no active DMA transactions in progress.
					if dma_busy_rclk = '0' and
					-- Transmit channels need no other qualification.
					((tx_rx_select = '0') or
					-- Receive channels must wait for all pending DMA transactions to complete.
					(tx_rx_select = '1' and dma_go_rclk = '0')) then
						ch_state <= idle;
					end if;
					
				when others  =>
						ch_state <= idle;
			end case;
        end if;
    end process;

--*************************************************** STATE MACHINE EVENTS *****************************************************

	process (sample_clk,rst)
    begin
		if rst='1' then
		
			synchronizer_off <= '0';
			pause_counter <= (others => '0');

        elsif rising_edge(sample_clk) then
        
			-- Set a sticky bit when the off strobe is detected during operation.
			if ch_state = idle then
				synchronizer_off <= '0';
			elsif synchronizer_on_off(1) = '1' then
				synchronizer_off <= '1';
			end if;
			
			-- The following code manages the pause counter.
			if (ch_state = initialize or ch_state = purge) and pause_counter /= x"F" then
				-- Increment the pause counter.
				pause_counter <= pause_counter + x"1";
			else
				-- Reset the pause counter.
				pause_counter <= (others => '0');
			end if;
			
        end if;
    end process;

--***************************************************** PORT ASSIGNMENTS *******************************************************
			
	process (sample_clk,rst)
    begin
		if rst='1' then
		
			stop_on_count <= '0';
			synchronizer_busy <= '0';
			channel_active <= '0';
			channel_ready <= '0';
			dma_disable <= '1';

        elsif rising_edge(sample_clk) then
        
			-- Generate a flag to indicate a "stop on count" request to the data throttle.
			if stop_event_rclk = "11" then stop_on_count <= '1'; else stop_on_count <= '0'; end if;
			
			-- Create a synchronizer busy flag that can be used to reset datapath elements.
			if ch_state = idle then synchronizer_busy <= '0'; else synchronizer_busy <= '1'; end if;
			
			-- Generate a channel active flag to enable datapath elements.
			if ch_state = stop_trig then channel_active <= '1'; else channel_active <= '0'; end if;

			-- Generate a channel ready flag when the synchronizer is waiting for a start trigger.
			if ch_state = start_trig then channel_ready <= '1'; else channel_ready <= '0'; end if;

			-- The DMA engine is always disabled during the idle state and also during the residue state for transmit channels.
			if ch_state = idle or (tx_rx_select = '0' and ch_state = residue) then dma_disable <= '1'; else dma_disable <= '0'; end if;
			
        end if;
    end process;

	-- The following signals must be reclocked so that they are coherent to the PCI clock.
	process (clk)
    begin
        if rising_edge(clk) then
        
        	dma_enable <= not(dma_disable);
        	synchronizer_status(0) <= synchronizer_busy;
         	synchronizer_status(1) <= channel_ready;
       
        end if;
    end process;
    
    -- Create a signal to clear the datapath while the synchronizer is idle.
    clear <= not(synchronizer_busy);

end structural;
